<?php
/**
 * Clips setups, checks and admin menu.
 * 
 * @package WP_Clips_Multisite/Includes
 * @subpackage Admin
 */


class WP_Clips_Admin {


	public function __construct() {

		// Check for multisite
		if( ! is_multisite() ) {
			add_action( 'admin_print_footer_scripts', function() {
				printf( '<div class="notice notice-error"><p><b>%s</b> %s <a href="https://wpclips.net/installing/" target="_blank">%s</a>.</p></div>',
					__( 'WP Clips Multisite', 'wp-clips' ),
					__( 'requires WordPress multisite. Enable multisite or delete the plugin and install', 'wp-clips' ),
					__( 'WP Clips', 'wp-clips' )
				);
			});
			return;
		}

		// Build Clips directory
		if( ! file_exists( WP_CLIPS_NETW ) ) $this->clips_builder();

		// Actions on upload and rebuild submits
		if( isset( $_POST[ 'wp-clips-upload' ] ) ) $this->clips_upload();	
		if( isset( $_POST[ 'wp-clips-rebuild' ] ) ) $this->rebuild();

		add_action( 'after_setup_theme', [ $this, 'clips_check' ], 4 );
		add_action( 'network_admin_menu', [ $this, 'clips_network_menu' ] );
		add_action( 'admin_menu', [ $this, 'clips_menu' ] );
		add_action( 'wp_clips_manager', [ $this, 'clips_control' ] );
		add_action( 'network_site_new_form', [ $this, 'clips_builder' ] );
	}

	/**
	 * Add menus to WordPress admin
	 * @since 3.0
	 */

	// Network menu
	public function clips_network_menu() {

		add_submenu_page(
				'settings.php',
				__( 'WP Clips Manager', 'wp-clips' ),
				__( 'Network Clips', 'wp-clips' ),
				'manage_options',
				'wp-clips-control',
				[ $this, 'clips_manager' ]
			);
	}

	// Site menu
	public function clips_menu() {

		add_options_page(
				__( 'WP Clips Manager', 'wp-clips' ),
				__( 'Clips', 'wp-clips' ),
				'manage_options',
				'wp-clips-control',
				[ $this, 'clips_manager' ]
			);
	}

	/**
	 * Install or update Precoded Clips
	 * @since 3.0
	 */
	public function clips_upload() {

		if( ! $_FILES[ 'zip_file' ][ 'name' ] ) return;

		$filename = $_FILES[ 'zip_file' ][ 'name' ];
		$source = $_FILES[ 'zip_file' ][ 'tmp_name' ];
		$type = $_FILES[ 'zip_file' ][ 'type' ];
		$accepted_types = [ 'application/zip', 'application/x-zip-compressed', 'multipart/x-zip', 'application/x-compressed' ];	

		// Destination before unzip
		$target = WP_CLIPS_ROOT . $filename;

		// Get current Clips
		$dir = $this->site()['path'];
		$dirs = [ $dir, $dir . '/precoded', $dir . '/unclipped' ];
		$clips = $this->clips( $dirs );

		// Check for compatible file
		$zip_check = in_array( $type, $accepted_types ) && substr( $filename, -4 ) == '.zip';
		$clip_check = substr( $filename, 0, 5 ) == 'clip-';

		// Isolate name and version
		$filebase = rtrim( $filename, '.zip' );
		$filedata = explode( '.', $filebase, 2 );
		$name = $filedata[0];
		$vers = isset( $filedata[1] ) ? $filedata[1] : false;

		$class = 'notice-error'; // Default

		// Get version of older file (if exists)
		if( $clip_check ) {
			foreach( $clips as $slug => $data ) {
				if( ! empty( $data['clipVers'] ) && $slug === $name )
					$current_vers = $data['clipVers'];
			}
		}

		// Check for approved file type
		if( ! $zip_check || ! $clip_check || ! $vers )
			$notice = __( 'The file is not a Clip or zip file. Please try again.', 'wp-clips' );

		// Check is newer version
		elseif( isset( $current_vers ) && version_compare( $vers, $current_vers, '<' ) )
			$notice = __( 'The source file is an older version and will not be installed', 'wp-clips' );

		// Check for PHP ZIP module
		elseif( ! class_exists( 'ZipArchive' ) )
			$notice = __( 'The server\'s PHP ZIP module must be enabled for uploads. Please contact your host.', 'wp-clips' );

		// Upload and extract file if checks
		elseif( move_uploaded_file( $source, $target ) ) {

			$zip = new ZipArchive();
			$x = $zip->open( $target );
			if( $x === true ) {

				$msg = __( 'was uploaded and installed', 'wp-clips' );
				$class = 'notice-success';
				$path = $dir . '/precoded/';

				if( $clip_check ) {

					// Cleanup
					foreach( $clips as $slug => $data ) 
						if( $slug === $name ) {
							$path = dirname( $data['clipPath'] );
							$this->delete( $data['clipPath'] );
							$msg = __( 'was updated', 'wp-clips' );
						}
				}

				$notice = sprintf( '<b>%s</b> %s.', $name, $msg );
				$zip->extractTo( $path );
				$zip->close();
				if( file_exists( $target ) ) unlink( $target );
			}
		}

		// Error if unsuccessful
		else $notice = __( 'There was a problem with the upload. Please try again.', 'wp-clips' );

		// Display notice
		add_action( 'admin_print_footer_scripts', function() use( $class, $notice ) {
			printf( '<div class="notice %s"><p>%s</p></div>', $class, $notice );
		});
	}

	/**
	 * Render Manager page
	 * @since 3.1.1
	 */
	public function clips_manager() {

		?>
		<div class="wrap">
			<h1><?php _e( 'WP Clips Manager', 'wp-clips' ); ?></h1>
            <p><?php
            	echo __( 'Multisite Plugin Version ', 'wp-clips' ) . WP_CLIPS_VERS;

				// Append update link (if new version exists)
				$xml_file = wp_remote_retrieve_body( wp_remote_get( 'https://wpclips.net/versions/wp-clips.xml' ) );
				$xml = simplexml_load_string( $xml_file );
				if( version_compare( $xml->vers, WP_CLIPS_VERS, '>' ) )
					printf( ' &#10141; <a href="https://wpclips.net/updating-wp-clips/">%s</a>', __( 'Update Available', 'wp-clips' ) );
			?></p><hr>
			<h3><?php _e( 'Install / Update Clips', 'wp-clips' ); ?></h3>

			<form enctype="multipart/form-data" method="post">
				<label><?php _e( 'Select a compatible Clip file (.zip):', 'wp-clips' ); ?></label>
				<input type="file" name="zip_file" />
				<?php submit_button( 'Install / Update', '', 'wp-clips-upload', true ); ?>
			</form>

            <?php do_action( 'wp_clips_manager' ); ?>
		</div>
    <?php
	}

	/**
	 * Add Clips Control
	 * @since 3.1
	 */
	public function clips_control() {
		
		?>
        <hr />
		<h3><?php _e( 'Control', 'wp-clips' ); ?></h3>
		<p><?php _e( 'Clips can be activated or unclipped. Precoded Clips can also be deleted after being unclipped.', 'wp-clips' ); ?></p>
		<?php

		// Get current blog id/name
		$id = $this->site()['id'];
		$dirs = [ __( 'Network', 'wp-clips' ) => WP_CLIPS_NETW ];
		if( $id ) {
			$name = get_blog_details( $id )->blogname . " ({$id})";
			$dirs[$name] = $this->site()['path'];
		}		
		?>        
		<form method="post">
			<ul><?php

				// List Clips with buttons
				foreach( $dirs as $name => $dir ) {

					printf( '<h4>%s</h4>', $name );

					if( ! $id || isset( $html ) ) { ?>
						<p>
							<span class="dashicons dashicons-yes"></span>
							<span class="dashicons dashicons-minus"></span>
							<span class="dashicons dashicons-no-alt"></span>
						</p><?php
					}

					// Get and sort Clips
					$clips = $this->clips([ $dir, $dir . '/precoded', $dir . '/unclipped' ]);
					ksort( $clips );
					$slug = empty( $clips['clip-custom'] ) ? 'clip-core' : 'clip-custom';
					$clips = [ $slug => $clips[$slug] ] + $clips;

					foreach( $clips as $slug => $data ) {

						extract( $data );
						$html = '<li>';

						foreach( [ 'active', 'unclipped', 'delete' ] as $btn ) {

							$disabled = ( $id ) && basename( $dir ) === 'network' ? ' disabled' : '';

							$html .= sprintf( '<input type="radio" name="%s-%s" value="%s"', $slug, basename( $dir ), $btn );
							switch( $btn ) {
								case 'active':
									$html .= $clipActv ? 'checked' : '';
									$text = __( 'Active', 'wp-clips' );
									break;
								case 'unclipped':
									$html .= $clipActv ? '' : 'checked';
									$text = __( 'Unclipped', 'wp-clips' );
									break;
								case 'delete':
									$text = __( 'Delete', 'wp-clips' );
									$disabled = empty( $clipVers ) || $clipActv || isset( $clipSafe ) ? ' disabled' : $disabled;
									$alert = __( 'Do you really want to delete this Clip', 'wp-clips' );
									$html .= sprintf( ' onclick="return confirm(%s%s%s)"', "'", $alert, "?'" );
							}
							$html .= sprintf( '%s> </input><span class="screen-reader-text">%s</span>', $disabled, $text );
						}

						$html .= $clipName;
						$html .= empty( $clipVers ) ? '' : sprintf( ' (v%s)', $clipVers );

						if( ! $id || $dir !== WP_CLIPS_NETW ) {

							if( ! empty( $authName ) ) {
								$html .= ! empty( $authPage ) ?
									sprintf( ' by <a href="%s" target="_blank">%s</a>', $authPage, $authName )
								:
									sprintf( ' by %s', $authName );
							}

							if( ! empty( $clipPage ) ) {
								$html .= sprintf( ' &#10141; <a href="%s" target="_blank">%s</a>', $clipPage, __( 'Guide', 'wp-clips' ) );
							}
						}

						unset( $clipVers, $clipPage, $clipSafe, $authName, $authPage );
						$html = apply_filters( 'wp_clips_manager_clip_meta', $html, $slug, $data );

						echo $html . '</li>';
					}
				}?>
			</ul>
			<?php submit_button( 'Rebuild Clips', 'primary', 'wp-clips-rebuild', true ); ?>   
		</form>
	<?php
	}

	/**
	 * Build Clips directories
	 * @since 3.0
	 */
	public function clips_builder() {

		// Build network Clips from template (if do not exist)
		$this->build( WP_CLIPS_NETW, '/clip-core' );

		// Setup site Clips from template (if do not exist)
		$ids = get_sites( [ 'fields' => 'ids' ] );
		foreach( $ids as $id ) {
			$this->build( WP_CLIPS_SITE . $id, '/clip-custom' );
		}
	}

	// Build Clips from template
	private function build( $site, $clip ) {

		// Build directories (if do not exist)
		if( ! file_exists( $site . '/precoded' ) )
			mkdir( $site . '/precoded' , 0755, true);
		if( ! file_exists( $site . '/unclipped' ) )
			mkdir( $site . '/unclipped' , 0755, true);

		// Build Clips from template (if do not exist)
		if( ! file_exists( $site . $clip ) &&
			! file_exists( $site . '/unclipped' . $clip )
		) {
			mkdir( $site . $clip, 0755, true);
			$tempfiles = WP_CLIPS_ROOT . 'templates' . $clip . '-tmpl/*';
			foreach( glob( $tempfiles ) as $file )
				copy( $file, $site . $clip . '/' . basename( $file ) );
		}
	}

	/**
	 * Check Clip compatibility
	 * @since 3.0
	 */
	public function clips_check() {

		$actv_theme = [ get_template(), is_child_theme() ? get_stylesheet() : '' ];
		$actv_plugins = [];
		$dir = $this->site()['path'];
		$dirs = [ $dir, $dir . '/precoded', $dir . '/unclipped' ];
		$plugins_netw = array_keys( get_site_option( 'active_sitewide_plugins') );
		$plugins_site = get_option( 'active_plugins' );

		foreach( array_merge( $plugins_netw, $plugins_site ) as $plugin ) {
			list( $plugin_name ) = explode( '/', $plugin );
			$actv_plugins[] = $plugin_name;
		}

		foreach( $this->clips( $dirs ) as $slug => $data ) {

			if( ! $data['clipActv'] ) continue;

			$notices = [];
			$name = $data['clipName'];
			$path = $data['clipPath'];

			// Check Clip is not a duplicate
			if( dirname( $path ) !== WP_CLIPS_NETW . '/precoded' &&
				file_exists( WP_CLIPS_NETW . '/precoded/' . $data['clipFile'] )
			) {
				$notices[] = __( 'is active on the network', 'wp-clips' );
				if( is_network_admin() ) $class = 'notice-info';
			}

			// Check if Clip is network only
			elseif( ! empty( $data['multiSit'] ) )
				if( ! $data['multiSit'] || dirname( $path, 2 ) != WP_CLIPS_NETW ) {
					$notices[] = __( 'requires network activation', 'wp-clips' );
				}

			// Check required plugins are active
			if( ! empty( $data['depsPlug'] ) ) {
				$plugins_deps = explode( ',', $data['depsPlug'] );
				$plugins_reqd = array_map( 'trim', $plugins_deps );
				foreach( $plugins_reqd as $plugin )
					if( ! in_array( $plugin, $actv_plugins ) ) {
						$notices[] = __( 'requires other plugins to be installed and active', 'wp-clips' );
						break;
					}
			}

			// Check theme is compatible
			if( ! empty( $data['depTheme'] ) )
				if( ! in_array( $data['depTheme'], $actv_theme ) ) {
					$notices[] = __( 'is not compatible with the current theme', 'wp-clips' );
				}

			// Unclip and alert if fail
			if( count( $notices ) > 0 ) {

				strpos( $path, '/precoded/' ) !== false ?
					rename( $path, str_replace( '/precoded/', '/unclipped/', $path ) )
				:
					rename( $path, dirname( $path ) . '/unclipped/' . $data['clipFile'] );

				foreach( $notices as $notice )
					add_action( 'admin_print_footer_scripts', function() use( $name, $notice ) {
						printf( '<div class="notice notice-error"><p><b>%s</b> %s. %s.</p></div>',
								$name, $notice, __( 'Refer to Clip documentation', 'wp-clips' )
							);
					});
			}
		}
	}

	/**
	 * Rebuild Clips directory
	 * @since 3.0
	 */
	private function rebuild() {

		$dir = $this->site()['path'];
		$dirs = [ $dir, $dir . '/precoded', $dir . '/unclipped' ];
		$ext = '-' . basename( $dir );

		foreach( $this->clips( $dirs ) as $slug => $data ) {

			if( ! isset( $_POST[ $slug . $ext ] ) ) continue;
			extract( $data );

			switch( $_POST[ $slug . $ext ] ) {

				// Move Clip to active dir if 'active' selected
				case 'active':
					if( ! $clipActv ) {
						$path = in_array( $clipFile, [ 'clip-core', 'clip-custom' ] ) ? '' : 'precoded/';
						rename( $clipPath, $dir . '/' . $path . $clipFile );
					}
					break;

				// Move Clip to unclipped dir if 'unclipped' selected
				case 'unclipped':
					if( $clipActv ) {
						rename( $clipPath, $dir . '/unclipped/' . $clipFile );
					}
					break;

				// Check uninstalls and delete Clip if 'delete' selected
				case 'delete':
					$uninstall = $clipPath . '/uninstall.php';
					if( file_exists( $uninstall ) ) include( $uninstall );
					$this->delete( $clipPath . '/' );
			}
		}

		add_action( 'admin_print_footer_scripts', function() {
			printf( '<div class="notice notice-success"><p>%s.</p></div>', __( 'Clips have been rebuilt', 'wp-clips' ) );
		});
	}

	// Get current site
	private function site() {

		$id = is_network_admin() ? 0 : get_current_blog_id();
		$path = ( $id ) ? WP_CLIPS_SITE . $id : WP_CLIPS_NETW;
		return [ 'id' => $id, 'path' => $path ];
	}

	// Get Clips and data
	private function clips( $dirs ) {

		$clips = [];

		foreach( $dirs as $dir ) {

			// Get Clips and data for selected directory
			foreach( glob( $dir . '/clip-*/', GLOB_NOSORT ) as $path ) {

				$filename = basename( $path );
				$filedata = explode( '.', $filename, 2 );

				$clip = [
					'clipPath' => untrailingslashit( $path ),
					'clipFile' => $filename,
					'clipActv' => basename( dirname( $path ) ) !== 'unclipped' ? true : false
				];

				if( file_exists( $path . 'core-functions.php' ) ) {
					$clip['clipName'] = __( 'Core Clip', 'wp-clips' );
				}
				else {

					$headers = [
						'clipName' => 'Clip Name',	
						'depTheme' => 'Required Theme',
						'depsPlug' => 'Required Plugins',
					];

					if( file_exists( $path . 'custom-functions.php' ) ) {
						$clip['clipName'] = __( 'Custom Clip', 'wp-clips' );
						$file = $path . 'custom-functions.php';
					}
					else {
						if( file_exists( $path . 'clip-functions.php' ) ) {
							$file = $path . 'clip-functions.php';
						}
						elseif( file_exists( $path . 'admin-functions.php' ) ) {
							$file = $path . 'admin-functions.php';
						}
						else continue;

						$headers = $headers + [
							'clipPage' => 'Clip URI',
							'clipVers' => 'Version',
							'authName' => 'Author',
							'authPage' => 'Author URI',
							'clipData' => 'Datafile',
							'clipEdit' => 'Editable',
							'clipSafe' => 'Safe',
							'multiSit' => 'Network'
						];
						$clip['clipName'] = $filedata[0];
						$clip['clipVers'] = empty( $filedata[1] ) ? '' : $filedata[1];
					}
					$data = $this->get_clip_data( $file, $headers );
					$clip = array_merge( $clip, array_filter( $data ) );
				}

				$clips[ $filedata[0] ] = $clip;
			}
		}

		return $clips;
	}

	// Get file headers
	private function get_clip_data( $file, $headers ) {

		$fp = fopen( $file, 'r' );
		$data = '';

		if( $fp ) {
			$data = fread( $fp, 1 * KB_IN_BYTES );
			fclose( $fp );
		}	 

		$data = str_replace( '\r', '\n', $data );

		foreach( $headers as $field => $regex )
			if( preg_match( '/^[ \t\/*#@]*' . preg_quote( $regex, '/' ) . ':(.*)$/mi', $data, $match ) && $match[1] ) {
				$headers[ $field ] = _cleanup_header_comment( $match[1] );
			}
			else $headers[ $field ] = '';

		return $headers;
	}

	// Delete directory and contents
	private function delete( $dir ) {

		$files = array_diff( scandir( $dir ), [ '.', '..' ] );
    	foreach( $files as $file ) {
			$path = $dir . '/' . $file;
			is_dir( $path ) ? $this->delete( $path ) : unlink( $path );
		}
		return rmdir( $dir );
	}
}

new WP_Clips_Admin();